/*!
 * Ext JS Library 3.1.1
 * Copyright(c) 2006-2010 Ext JS, LLC
 * licensing@extjs.com
 * http://www.extjs.com/license
 */
/**
 * @class Ext.ux.Spinner
 * @extends Ext.util.Observable
 * Creates a Spinner control utilized by Ext.ux.form.SpinnerField
 */
Ext.ux.Spinner = Ext.extend(Ext.util.Observable, {
  incrementValue: 1,
  alternateIncrementValue: 5,
  triggerClass: 'x-form-spinner-trigger',
  alternateKey: Ext.EventObject.shiftKey,
  defaultValue: 0,
  accelerate: false,
  zeroPad: false,
  padLength: false,

  constructor: function(config) {
    Ext.ux.Spinner.superclass.constructor.call(this, config);
    Ext.apply(this, config);
    this.mimicing = false;
  },
  init: function(field) {
    this.field = field;

    field.afterMethod('onRender', this.doRender, this);
    field.afterMethod('onEnable', this.doEnable, this);
    field.afterMethod('onDisable', this.doDisable, this);
    field.afterMethod('afterRender', this.doAfterRender, this);
    field.afterMethod('onResize', this.doResize, this);
    field.afterMethod('onFocus', this.doFocus, this);
    field.beforeMethod('onDestroy', this.doDestroy, this);
  },
  doRender: function(ct, position) {
    var el = this.el = this.field.getEl();
    var f = this.field;

    if (!f.wrap) {
      f.wrap = this.wrap = el.wrap({
        cls: 'x-form-field-wrap'
      });
    }
    else {
      this.wrap = f.wrap.addClass('x-form-field-wrap');
    }

    this.trigger = this.wrap.createChild({
      tag: 'img',
      src: Ext.BLANK_IMAGE_URL,
      cls: 'x-form-trigger ' + this.triggerClass
    });

    if (!f.width) {
      this.wrap.setWidth(el.getWidth() + this.trigger.getWidth());
    }

    this.initTrigger();
    this.initSpinner();
  },
  doAfterRender: function() {
    var y;
    if (Ext.isIE && this.el.getY() != (y = this.trigger.getY())) {
      this.el.position();
      this.el.setY(y);
    }
  },
  doEnable: function() {
    if (this.wrap) {
      this.wrap.removeClass(this.field.disabledClass);
    }
  },
  doDisable: function() {
    if (this.wrap) {
      this.wrap.addClass(this.field.disabledClass);
      this.el.removeClass(this.field.disabledClass);
    }
  },
  doResize: function(w, h) {
    if (typeof w == 'number') {
      this.el.setWidth(w - this.trigger.getWidth());
    }
    this.wrap.setWidth(this.el.getWidth() + this.trigger.getWidth());
  },
  doFocus: function() {
    if (!this.mimicing) {
      this.wrap.addClass('x-trigger-wrap-focus');
      this.mimicing = true;
      Ext.get(Ext.isIE ? document.body : document).on('mousedown', this.mimicBlur, this, {
        delay: 10
      });
      this.el.on('keydown', this.checkTab, this);
    }
  },
  checkTab: function(e) {
    if (e.getKey() == e.TAB) {
      this.triggerBlur();
    }
  },
  mimicBlur: function(e) {
    if (!this.wrap.contains(e.target) && this.field.validateBlur(e)) {
      this.triggerBlur();
    }
  },
  triggerBlur: function() {
    this.mimicing = false;
    Ext.get(Ext.isIE ? document.body : document).un('mousedown', this.mimicBlur, this);
    this.el.un('keydown', this.checkTab, this);
    this.field.beforeBlur();
    this.wrap.removeClass('x-trigger-wrap-focus');
    this.field.onBlur.call(this.field);
  },
  initTrigger: function() {
    this.trigger.addClassOnOver('x-form-trigger-over');
    this.trigger.addClassOnClick('x-form-trigger-click');
  },
  initSpinner: function() {
    this.field.addEvents({
      'spin': true,
      'spinup': true,
      'spindown': true
    });

    this.keyNav = new Ext.KeyNav(this.el, {
      'up': function(e) {
        e.preventDefault();
        this.onSpinUp();
      },

      'down': function(e) {
        e.preventDefault();
        this.onSpinDown();
      },

      'pageUp': function(e) {
        e.preventDefault();
        this.onSpinUpAlternate();
      },

      'pageDown': function(e) {
        e.preventDefault();
        this.onSpinDownAlternate();
      },

      scope: this
    });

    this.repeater = new Ext.util.ClickRepeater(this.trigger, {
      accelerate: this.accelerate
    });

    this.field.mon(this.repeater, 'click', this.onTriggerClick, this, {
      preventDefault: true
    });

    this.field.mon(this.trigger, {
      mouseover: this.onMouseOver,
      mouseout: this.onMouseOut,
      mousemove: this.onMouseMove,
      mousedown: this.onMouseDown,
      mouseup: this.onMouseUp,
      scope: this,
      preventDefault: true
    });

    this.field.mon(this.wrap, 'mousewheel', this.handleMouseWheel, this);
  },
  onMouseOver: function() {
    if (this.disabled) {
      return;
    }
    var middle = this.getMiddle();
    this.tmpHoverClass = (Ext.EventObject.getPageY() < middle) ? 'x-form-spinner-overup' : 'x-form-spinner-overdown';
    this.trigger.addClass(this.tmpHoverClass);
  },
  onMouseOut: function() {
    this.trigger.removeClass(this.tmpHoverClass);
  },
  onMouseMove: function() {
  },
  onMouseDown: function() {
    if (this.disabled) {
      return;
    }
    var middle = this.getMiddle();
    this.tmpClickClass = (Ext.EventObject.getPageY() < middle) ? 'x-form-spinner-clickup' : 'x-form-spinner-clickdown';
    this.trigger.addClass(this.tmpClickClass);
  },
  onMouseUp: function() {
    this.trigger.removeClass(this.tmpClickClass);
  },
  onTriggerClick: function() {
    if (this.disabled || this.el.dom.readOnly) {
      return;
    }
    var middle = this.getMiddle();
    var ud = (Ext.EventObject.getPageY() < middle) ? 'Up' : 'Down';
    this['onSpin' + ud]();
  },
  getMiddle: function() {
    var t = this.trigger.getTop();
    var h = this.trigger.getHeight();
    var middle = t + (h / 2);
    return middle;
  },
  isSpinnable: function() {
    if (this.disabled || this.el.dom.readOnly) {
      Ext.EventObject.preventDefault();
      return false;
    }

    return true;
  },
  handleMouseWheel: function(e) {
    if (this.wrap.hasClass('x-trigger-wrap-focus') == false) {
      return;
    }

    var delta = e.getWheelDelta();
    if (delta > 0) {
      this.onSpinUp();
      e.stopEvent();
    }
    else if (delta < 0) {
      this.onSpinDown();
      e.stopEvent();
    }
  },
  onSpinUp: function() {
    if (this.isSpinnable() == false) {
      return;
    }
    if (Ext.EventObject.shiftKey == true) {
      this.onSpinUpAlternate();
      return;
    }
    else {
      this.spin(false, false);
    }
    this.field.fireEvent('spin', this);
    this.field.fireEvent('spinup', this);
  },
  onSpinDown: function() {
    if (this.isSpinnable() == false) {
        return;
    }
    if (Ext.EventObject.shiftKey == true) {
      this.onSpinDownAlternate();
      return;
    }
    else {
      this.spin(true, false);
    }
    this.field.fireEvent('spin', this);
    this.field.fireEvent('spindown', this);
  },
  onSpinUpAlternate: function() {
    if (this.isSpinnable() == false) {
      return;
    }
    this.spin(false, true);
    this.field.fireEvent('spin', this);
    this.field.fireEvent('spinup', this);
  },
  onSpinDownAlternate: function() {
    if (this.isSpinnable() == false) {
      return;
    }
    this.spin(true, true);
    this.field.fireEvent('spin', this);
    this.field.fireEvent('spindown', this);
  },
  spin: function(down, alternate) {
    var v = parseFloat(this.field.getValue());
    var incr = (alternate == true) ? this.alternateIncrementValue : this.incrementValue;
    (down == true) ? v -= incr : v += incr;

    this.updateValue(v);
  },
  updateValue: function(v, b) {
    if (isNaN(v)) {
      v = this.defaultValue;
    }

    if (!b) {
      v = this.fixBoundries(v);
      v = this.addPadding(v);
    }

    this.field.setRawValue(v);
  },
  addPadding: function(value) {
    if (this.zeroPad && this.padLength) {
      var l = value.toString().length;
      if (l < this.padLength) {
        value = new Array(this.padLength - l + 1).join('0') + value;
      }
    }

    return value;
  },
  fixBoundries: function(value) {
    var v = value;

    if (this.field.minValue != undefined && v < this.field.minValue) {
      v = this.field.minValue;
    }
    if (this.field.maxValue != undefined && v > this.field.maxValue) {
      v = this.field.maxValue;
    }

    return this.fixPrecision(v);
  },
  fixPrecision: function(value) {
    var nan = isNaN(value);
    if (!this.field.allowDecimals || this.field.decimalPrecision == -1 || nan || !value) {
      return nan ? '' : value;
    }
    return parseFloat(parseFloat(value).toFixed(this.field.decimalPrecision));
  },
  doDestroy: function() {
    if (this.trigger) {
      this.trigger.remove();
    }

    if (this.wrap) {
      this.wrap.remove();
      delete this.field.wrap;
    }

    if (this.repeater) {
      this.repeater.purgeListeners();
    }
  }
});

Ext.form.Spinner = Ext.ux.Spinner;

/*!
 * Ext JS Library 3.1.1
 * Copyright(c) 2006-2010 Ext JS, LLC
 * licensing@extjs.com
 * http://www.extjs.com/license
 */
Ext.ns('Ext.ux.form');

/**
 * @class Ext.ux.form.SpinnerField
 * @extends Ext.form.NumberField
 * Creates a field utilizing Ext.ux.Spinner
 * @xtype spinnerfield
 */
Ext.ux.form.SpinnerField = Ext.extend(Ext.form.NumberField, {
  actionMode: 'wrap',
  deferHeight: true,
  autoSize: Ext.emptyFn,
  onBlur: Ext.emptyFn,
  adjustSize: Ext.BoxComponent.prototype.adjustSize,

  constructor: function(config) {
    var spinnerConfig = Ext.copyTo({}, config, 'incrementValue,alternateIncrementValue,accelerate,defaultValue,triggerClass,zeroPad,padLength');
    var spl = this.spinner = new Ext.ux.Spinner(spinnerConfig);

    var plugins = config.plugins
      ? (Ext.isArray(config.plugins)
        ? config.plugins.push(spl)
        : [config.plugins, spl])
      : spl;

    Ext.ux.form.SpinnerField.superclass.constructor.call(this, Ext.apply(config, {plugins: plugins}));
  },
  getResizeEl: function() {
    return this.wrap;
  },
  getPositionEl: function() {
    return this.wrap;
  },
  alignErrorIcon: function() {
    if (this.wrap) {
      this.errorIcon.alignTo(this.wrap, 'tl-tr', [2, 0]);
    }
  },
  validateBlur: function() {
    return true;
  },
  setValue: function(v) {
    this.spinner.updateValue(v, this.allowBlank);
  }
});

Ext.reg('spinnerfield', Ext.ux.form.SpinnerField);
Ext.form.SpinnerField = Ext.ux.form.SpinnerField;

/*
 * PagingStore for Ext 3.2 - v0.5
 */
Ext.ns('Ext.ux.data');
Ext.ux.data.PagingStore = Ext.extend(Ext.data.Store, {
  add: function (records) {
    records = [].concat(records);
    if (records.length < 1) {
      return;
    }

    for (var i = 0, len = records.length; i < len; i++) {
      records[i].join(this);
    }

    var index = this.data.length;
    this.data.addAll(records);

    if (this.allData) {
      this.allData.addAll(records);
    }

    if (this.snapshot) {
      this.snapshot.addAll(records);
    }

    this.totalLength += records.length;
  },
  remove: function (record) {
    if (Ext.isArray(record)) {
      Ext.each(record, function (r) {
        this.remove(r);
      }, this);
      return;
    }

    if (this != record.store) {
      return;
    }

    record.join(null);

    var index = this.data.indexOf(record);
    if (index > -1) {
      this.data.removeAt(index);
    }
    if (this.pruneModifiedRecords) {
      this.modified.remove(record);
    }

    if (this.allData) {
      this.allData.remove(record);
    }

    if (this.snapshot) {
      this.snapshot.remove(record);
    }

    this.totalLength--;

    if (index > -1) {
      this.fireEvent('remove', this, record, index);
    }
  },
  removeAll: function (silent) {
    var items = [].concat((this.snapshot || this.allData || this.data).items);
    this.clearData();

    if (this.pruneModifiedRecords) {
      this.modified = [];
    }

    this.totalLength = 0;

    if (silent !== true) {
      this.fireEvent('clear', this, items);
    }
  },
  insert: function (index, records) {
    records = [].concat(records);
    for (var i = 0, len = records.length; i < len; i++) {
      this.data.insert(index, records[i]);
      records[i].join(this);
    }

    if (this.allData) {
      this.allData.addAll(records);
    }

    if (this.snapshot) {
      this.snapshot.addAll(records);
    }

    this.totalLength += records.length;
  },
  getById: function (id) {
    return (this.snapshot || this.allData || this.data).key(id);
  },
  clearData: function () {
    if (this.allData) {
      this.data = this.allData;
      delete this.allData;
    }

    if (this.snapshot) {
      this.data = this.snapshot;
      delete this.snapshot;
    }

    this.data.each(function (rec) {
      rec.join(null);
    });

    this.data.clear();
  },
  execute: function (action, rs, options, batch) {
    if (!Ext.data.Api.isAction(action)) {
      throw new Ext.data.Api.Error('execute', action);
    }

    options = Ext.applyIf(options || {}, {
      params: {}
    });

    if (batch !== undefined) {
      this.addToBatch(batch);
    }

    var doRequest = true;
    if (action === 'read') {
      doRequest = this.fireEvent('beforeload', this, options);
      Ext.applyIf(options.params, this.baseParams);
    }
    else {
      if (this.writer.listful === true && this.restful !== true) {
        rs = (Ext.isArray(rs)) ? rs : [rs];
      }
      else if (Ext.isArray(rs) && rs.length == 1) {
        rs = rs.shift();
      }

      if ((doRequest = this.fireEvent('beforewrite', this, action, rs, options)) !== false) {
        this.writer.apply(options.params, this.baseParams, action, rs);
      }
    }

    if (doRequest !== false) {
      if (this.writer && this.proxy.url && !this.proxy.restful && !Ext.data.Api.hasUniqueUrl(this.proxy, action)) {
        options.params.xaction = action;
      }

      if (action === 'read' && this.isPaging(Ext.apply({}, options.params))) {
        (function () {
          if (this.allData) {
            this.data = this.allData;
            delete this.allData;
          }

          this.applyPaging();
          this.fireEvent('datachanged', this);
          var r = [].concat(this.data.items);
          this.fireEvent('load', this, r, options);

          if (options.callback) {
            options.callback.call(options.scope || this, r, options, true);
          }
        }).defer(1, this);

        return true;
      }

      this.proxy.request(Ext.data.Api.actions[action], rs, options.params, this.reader, this.createCallback(action, rs, batch), this, options);
    }

    return doRequest;
  },
  loadRecords: function (o, options, success) {
    if (this.isDestroyed === true) {
      return;
    }

    if (!o || success === false) {
      if (success !== false) {
        this.fireEvent('load', this, [], options);
      }
      if (options.callback) {
        options.callback.call(options.scope || this, [], options, false, o);
      }

      return;
    }

    var r = o.records, t = o.totalRecords || r.length;
    if (!options || options.add !== true) {
      if (this.pruneModifiedRecords) {
        this.modified = [];
      }
      for (var i = 0, len = r.length; i < len; i++) {
        r[i].join(this);
      }

      this.clearData();
      this.data.addAll(r);
      this.totalLength = t;
      this.applySort();

      if (!this.allData) {
        this.applyPaging();
      }

      if (r.length > this.getCount()) {
        r = [].concat(this.data.items);
      }

      this.fireEvent('datachanged', this);
    }
    else {
      this.totalLength = Math.max(t, this.data.length + r.length);
      this.add(r);
    }

    this.fireEvent('load', this, r, options);
    if (options.callback) {
      options.callback.call(options.scope || this, r, options, true);
    }
  },
  loadData: function (o, append) {
    this.isPaging(Ext.apply({}, this.lastOptions ? this.lastOptions.params : null, this.baseParams));

    var r = this.reader.readRecords(o);
    this.loadRecords(r, {
      add: append
    }, true);
  },
  getTotalCount: function () {
    if (this.allData) {
      return this.allData.getCount();
    }

    return this.totalLength || 0;
  },
  sortData: function () {
    var sortInfo = this.hasMultiSort ? this.multiSortInfo : this.sortInfo,
        direction = sortInfo.direction || 'ASC', sorters = sortInfo.sorters, sortFns = [];

    if (!this.hasMultiSort) {
      sorters = [{
        direction: direction,
        field: sortInfo.field
      }];
    }

    for (var i = 0, j = sorters.length; i < j; i++) {
      sortFns.push(this.createSortFunction(sorters[i].field, sorters[i].direction));
    }

    if (!sortFns.length) {
      return;
    }

    var directionModifier = direction.toUpperCase() == 'DESC' ? -1 : 1;
    var fn = function (r1, r2) {
      var result = sortFns[0].call(this, r1, r2);
      if (sortFns.length > 1) {
        for (var i = 1, j = sortFns.length; i < j; i++) {
          result = result || sortFns[i].call(this, r1, r2);
        }
      }

      return directionModifier * result;
    };

    if (this.allData) {
      this.data = this.allData;
      delete this.allData;
    }

    this.data.sort(direction, fn);
    if (this.snapshot && this.snapshot != this.data) {
      this.snapshot.sort(direction, fn);
    }

    this.applyPaging();
  },
  filterBy: function (fn, scope) {
    this.snapshot = this.snapshot || this.allData || this.data;
    this.data = this.queryBy(fn, scope || this);
    this.applyPaging();

    this.fireEvent('datachanged', this);
  },
  clearFilter: function (suppressEvent) {
    if (this.isFiltered()) {
      this.data = this.snapshot;
      delete this.snapshot;
      delete this.allData;
      this.applyPaging();

      if (suppressEvent !== true) {
        this.fireEvent('datachanged', this);
      }
    }
  },
  isFiltered: function () {
    return !!this.snapshot && this.snapshot != (this.allData || this.data);
  },
  queryBy: function (fn, scope) {
    var data = this.snapshot || this.allData || this.data;
    return data.filterBy(fn, scope || this);
  },
  collect: function (dataIndex, allowNull, bypassFilter) {
    var d = (bypassFilter === true ? this.snapshot || this.allData || this.data : this.data).items;
    var v, sv, r = [], l = {};
    for (var i = 0, len = d.length; i < len; i++) {
      v = d[i].data[dataIndex];
      sv = String(v);
      if ((allowNull || !Ext.isEmpty(v)) && !l[sv]) {
        l[sv] = true;
        r[r.length] = v;
      }
    }
    return r;
  },
  findInsertIndex: function(record) {
    this.suspendEvents();
    var data = this.data.clone();
    this.data.add(record);
    this.applySort();
    var index = this.data.indexOf(record);
    this.data = data;
    this.totalLength--;
    this.resumeEvents();
    return index;
  },
  isPaging: function (params) {
    var pn = this.paramNames, start = params[pn.start], limit = params[pn.limit];
    if ((typeof start != 'number') || (typeof limit != 'number')) {
      delete this.start;
      delete this.limit;
      this.lastParams = params;
      return false;
    }
    this.start = start;
    this.limit = limit;
    delete params[pn.start];
    delete params[pn.limit];
    var lastParams = this.lastParams;
    this.lastParams = params;

    if (!this.proxy) {
      return true;
    }

    if (!lastParams) {
      return false;
    }

    for (var param in params) {
      if (params.hasOwnProperty(param) && (params[param] !== lastParams[param])) {
        return false;
      }
    }

    for (param in lastParams) {
      if (lastParams.hasOwnProperty(param) && (params[param] !== lastParams[param])) {
        return false;
      }
    }

    return true;
  },
  applyPaging: function () {
    var start = this.start, limit = this.limit;

    if ((typeof start == 'number') && (typeof limit == 'number')) {
      var allData = this.data, data = new Ext.util.MixedCollection(allData.allowFunctions, allData.getKey);
      data.items = allData.items.slice(start, start + limit);
      data.keys = allData.keys.slice(start, start + limit);
      var len = data.length = data.items.length;
      var map = {};
      for (var i = 0; i < len; i++) {
        var item = data.items[i];
        map[data.getKey(item)] = item;
      }
      data.map = map;
      this.allData = allData;
      this.data = data;
    }
  }
});

Ext.ux.data.PagingJsonStore = Ext.extend(Ext.ux.data.PagingStore, {
  constructor: function(config) {
    Ext.data.JsonStore.prototype.constructor.call(this, config);
  }
});
Ext.reg('pagingjsonstore', Ext.ux.data.PagingJsonStore);

Ext.ux.PagingToolbar = Ext.extend(Ext.PagingToolbar, {
  prevCursor: 0,
  onLoad: function (store, r, o) {
    if (!this.rendered) {
      this.dsLoaded = [store, r, o];
      return;
    }

    var p = this.getParams();
    this.cursor = (o.params && o.params[p.start] !== undefined) ? o.params[p.start] : this.prevCursor;
    this.onChange();
  },
  onChange: function () {
    var t = this.store.getTotalCount(), s = this.pageSize;
    if (this.cursor >= t) {
      var x = t > 0 ? (t - 1) : 0;
      this.cursor = Math.floor(t / s) * s;
    }

    var d = this.getPageData(), ap = d.activePage, ps = d.pages;

    this.afterTextItem.setText(String.format(this.afterPageText, d.pages));
    this.inputItem.enable();
    this.inputItem.setValue(ap);
    this.first.setDisabled(ap == 1);
    this.prev.setDisabled(ap == 1);
    this.next.setDisabled(ap == ps);
    this.last.setDisabled(ap == ps);
    this.refresh.enable();
    this.updateInfo();
    this.fireEvent('change', this, d);

    this.prevCursor = this.cursor;
  },
  onClear: function () {
    this.cursor = 0;
    this.onChange();
  },
  disableAll: function() {
    this.inputItem.disable();
    this.first.disable();
    this.prev.disable();
    this.next.disable();
    this.last.disable();
    this.refresh.disable();
  },
  doRefresh: function () {
    delete this.store.lastParams;
    this.disableAll();
    this.doLoad(this.cursor);
  },
  bindStore: function (store, initial) {
    var doLoad;

    if (!initial && this.store) {
      if (store !== this.store && this.store.autoDestroy) {
        this.store.destroy();
      }
      else {
        this.store.un('beforeload', this.beforeLoad, this);
        this.store.un('load', this.onLoad, this);
        this.store.un('exception', this.onLoadError, this);
        this.store.un('datachanged', this.onChange, this);
        this.store.un('add', this.onChange, this);
        this.store.un('remove', this.onChange, this);
        this.store.un('clear', this.onClear, this);
      }

      if (!store) {
        this.store = null;
      }
    }

    if (store) {
      store = Ext.StoreMgr.lookup(store);
      store.on({
        scope: this,
        beforeload: this.beforeLoad,
        load: this.onLoad,
        exception: this.onLoadError,
        datachanged: this.onChange,
        add: this.onChange,
        remove: this.onChange,
        clear: this.onClear
      });

      doLoad = true;
    }

    this.store = store;
    if (doLoad) {
      this.onLoad(store, null, {});
    }
  }
});

Ext.ns('Ext.ux.grid');

Ext.ux.grid.CheckColumn = Ext.extend(Ext.grid.Column, {
  processEvent: function(name, e, grid, rowIndex, colIndex ) {
    var record = grid.store.getAt(rowIndex), value = !record.data[ this.dataIndex ];
    record.set(this.dataIndex, value);

    if (this.callback) {
      this.callback(record.data.id, this.dataIndex, value);
    }

    return false;
  },
  renderer: function(v, p, record) {
    if (this.getValue) {
      v = this.getValue(record);

      if (typeof v == 'undefined') {
        return '';
      }
    }

    if (this.invert) {
      v = !v;
    }

    p.css += ' x-grid3-check-col-td';
    return String.format('<div class="x-grid3-check-col{0}">&#160;</div>', v ? '-on' : '');
  }
});

Ext.preg('checkcolumn', Ext.ux.grid.CheckColumn);
Ext.grid.CheckColumn = Ext.ux.grid.CheckColumn;
Ext.grid.Column.types.checkcolumn = Ext.ux.grid.CheckColumn;

Ext.ux.ColorField = Ext.extend(Ext.form.TriggerField, {
  triggerConfig: {
    src: Ext.BLANK_IMAGE_URL,
    tag: 'img',
    cls: 'x-form-trigger x-form-color-trigger'
  },
  invalidText: 'Colors must be in a the hex format #FFFFFF.',
  regex: /^\#[0-9A-F]{6}$/i,
  allowBlank: false,
  initComponent: function() {
    Ext.ux.ColorField.superclass.initComponent.call(this);
    this.addEvents('select');
    this.on('change', function(c, v) {
      this.onSelect(c, v);
    }, this);
  },
  onDestroy: function() {
    Ext.destroy(this.menu);
    Ext.ux.ColorField.superclass.onDestroy.call(this);
  },
  afterRender: function() {
    Ext.ux.ColorField.superclass.afterRender.call(this);
    this.detectFontColor();
  },
  onTriggerClick: function() {
    if (!this.disabled) {
      if (this.menu == null) {
        this.menu = new Ext.ux.ColorMenu();
      }

      this.onFocus();
      this.menu.picker.setValue(this.getValue() || '#000000');
      this.menu.show(this.el, 'tl-bl?');
      this.menuEvents('on');
    }
  },
  menuEvents: function(method) {
    this.menu[ method ]('select', this.onSelect, this);
    this.menu[ method ]('hide', this.onMenuHide, this);
    this.menu[ method ]('show', this.onFocus, this);
  },
  onSelect: function(m, d) {
    this.setValue(d);
    this.fireEvent('select', this, d);
    this.detectFontColor();
  },
  detectFontColor: function() {
    var value;

    if (this.menu && this.menu.picker.rawValue) {
      value = this.menu.picker.rawValue;
    }
    else if (!this.value) {
      value = '000000';
    }
    else {
      value = this.value;
    }

    var h2d = function(d) { return parseInt(d, 16); }

    var avg = (h2d(value.slice(1, 3)) + h2d(value.slice(3, 5)) + h2d(value.slice(5))) / 3;

    if (this.el) {
      this.el.setStyle('background', value);
      this.el.setStyle('color', (avg > 128) ? '#000' : '#FFF');
    }
  },
  setValue: function(value) {
    Ext.ux.ColorField.superclass.setValue.call(this, value);
    this.detectFontColor();
  },
  onMenuHide: function() {
    this.focus(false, 60);
    this.menuEvents('un');
  }
});

Ext.ux.ColorMenu = Ext.extend(Ext.menu.Menu, {
  enableScrolling: false,
  initComponent: function() {
    this.picker = new Ext.ux.ColorPicker(this.initialConfig);

    Ext.apply(this, {
      plain: true,
      showSeparator: false,
      items: this.picker
    });

    this.picker.purgeListeners();
    Ext.ux.ColorMenu.superclass.initComponent.call(this);
    this.relayEvents(this.picker, ['select']);

    this.on('select', this.menuHide, this);
    if  (this.handler) {
      this.on('select', this.handler, this.scope || this);
    }
  },
  menuHide: function() {
    this.hide(true);
  },
  doLayout: function(shallow, force) {
    Ext.ux.ColorMenu.superclass.doLayout.call(this, shallow, force);
    this.getEl().setZIndex(30000);
  }
});

Ext.ux.ColorPicker = function(config) {
  Ext.ux.ColorPicker.superclass.constructor.call(this, config);
  this.addEvents('select');

  if (!this.value) {
    this.value = this.defaultValue;
  }

  if (this.handler) {
    this.on('select', this.handler, this.scope, true);
  }
};

Ext.extend(Ext.ux.ColorPicker, Ext.ColorPalette, {
  itemCls: 'x-color-picker',
  defaultValue: '#000000',
  width: 200,
  onRender: function(container, position) {
    if (!this.value) {
      this.value = this.defaultValue;
    }

    this.itemCls = 'x-color-palette';
    Ext.ux.ColorPicker.superclass.onRender.call(this, container, position);
  },
  afterRender: function() {
    Ext.ColorPalette.superclass.afterRender.call(this);
  },
  select: function(e, t) {
    this.value = e;
    Ext.ux.ColorPicker.superclass.select.call(this, e);
    this.fireEvent('select', this, '#' + this.value);
  },
  getValue: function() {
    return this.value;
  },
  setValue: function(v) {
    this.value = v;
  }
});
Ext.reg('colorfield', Ext.ux.ColorField);
